@@ -1,8 +1,6 @@
-extern crate ansi_term;
-
 use crate::display_node::DisplayNode;
 
-use self::ansi_term::Colour::Red;
+use ansi_term::Colour::Red;
 use lscolors::{LsColors, Style};
 
 use unicode_width::UnicodeWidthStr;
@@ -16,7 +14,7 @@ use std::iter::repeat;
 use std::path::Path;
 use thousands::Separable;
 
-static UNITS: [char; 4] = ['T', 'G', 'M', 'K'];
+pub static UNITS: [char; 4] = ['T', 'G', 'M', 'K'];
 static BLOCKS: [char; 5] = ['█', '▓', '▒', '░', ' '];
 
 pub struct DisplayData {
@@ -28,6 +26,7 @@ pub struct DisplayData {
     pub base_size: u64,
     pub longest_string_length: usize,
     pub ls_colors: LsColors,
+    pub iso: bool,
 }
 
 impl DisplayData {
@@ -110,35 +109,45 @@ pub fn draw_it(
     use_full_path: bool,
     is_reversed: bool,
     no_colors: bool,
-    no_percents: bool,
+    no_percent_bars: bool,
     terminal_width: usize,
     by_filecount: bool,
-    option_root_node: Option<DisplayNode>,
+    root_node: &DisplayNode,
+    iso: bool,
+    skip_total: bool,
 ) {
-    if option_root_node.is_none() {
-        return;
-    }
-    let root_node = option_root_node.unwrap();
+    let biggest = match skip_total {
+        false => root_node,
+        true => root_node
+            .get_children_from_node(false)
+            .next()
+            .unwrap_or(root_node),
+    };
 
     let num_chars_needed_on_left_most = if by_filecount {
-        let max_size = root_node.size;
+        let max_size = biggest.size;
         max_size.separate_with_commas().chars().count()
     } else {
-        5 // Under normal usage we need 5 chars to display the size of a directory
+        find_biggest_size_str(root_node, iso)
     };
 
-    let terminal_width = terminal_width - 9 - num_chars_needed_on_left_most;
+    assert!(
+        terminal_width > num_chars_needed_on_left_most + 2,
+        "Not enough terminal width"
+    );
+
+    let allowed_width = terminal_width - num_chars_needed_on_left_most - 2;
     let num_indent_chars = 3;
     let longest_string_length =
-        find_longest_dir_name(&root_node, num_indent_chars, terminal_width, !use_full_path);
+        find_longest_dir_name(root_node, num_indent_chars, allowed_width, !use_full_path);
 
-    let max_bar_length = if no_percents || longest_string_length >= terminal_width as usize {
+    let max_bar_length = if no_percent_bars || longest_string_length + 7 >= allowed_width as usize {
         0
     } else {
-        terminal_width as usize - longest_string_length
+        allowed_width as usize - longest_string_length - 7
     };
 
-    let first_size_bar = repeat(BLOCKS[0]).take(max_bar_length).collect::<String>();
+    let first_size_bar = repeat(BLOCKS[0]).take(max_bar_length).collect();
 
     let display_data = DisplayData {
         short_paths: !use_full_path,
@@ -146,16 +155,37 @@ pub fn draw_it(
         colors_on: !no_colors,
         by_filecount,
         num_chars_needed_on_left_most,
-        base_size: root_node.size,
+        base_size: biggest.size,
         longest_string_length,
         ls_colors: LsColors::from_env().unwrap_or_default(),
+        iso,
     };
     let draw_data = DrawData {
         indent: "".to_string(),
         percent_bar: first_size_bar,
         display_data: &display_data,
     };
-    display_node(root_node, &draw_data, true, true);
+
+    if !skip_total {
+        display_node(root_node, &draw_data, true, true);
+    } else {
+        for (count, c) in root_node
+            .get_children_from_node(draw_data.display_data.is_reversed)
+            .enumerate()
+        {
+            let is_biggest = display_data.is_biggest(count, root_node.num_siblings());
+            let was_i_last = display_data.is_last(count, root_node.num_siblings());
+            display_node(c, &draw_data, is_biggest, was_i_last);
+        }
+    }
+}
+
+fn find_biggest_size_str(node: &DisplayNode, iso: bool) -> usize {
+    let mut mx = human_readable_number(node.size, iso).chars().count();
+    for n in node.children.iter() {
+        mx = max(mx, find_biggest_size_str(n, iso));
+    }
+    mx
 }
 
 fn find_longest_dir_name(
@@ -177,26 +207,20 @@ fn find_longest_dir_name(
         .fold(longest, max)
 }
 
-fn display_node(node: DisplayNode, draw_data: &DrawData, is_biggest: bool, is_last: bool) {
+fn display_node(node: &DisplayNode, draw_data: &DrawData, is_biggest: bool, is_last: bool) {
     // hacky way of working out how deep we are in the tree
     let indent = draw_data.get_new_indent(!node.children.is_empty(), is_last);
     let level = ((indent.chars().count() - 1) / 2) - 1;
-    let bar_text = draw_data.generate_bar(&node, level);
+    let bar_text = draw_data.generate_bar(node, level);
 
-    let to_print = format_string(
-        &node,
-        &*indent,
-        &*bar_text,
-        is_biggest,
-        draw_data.display_data,
-    );
+    let to_print = format_string(node, &indent, &bar_text, is_biggest, draw_data.display_data);
 
     if !draw_data.display_data.is_reversed {
         println!("{}", to_print)
     }
 
     let dd = DrawData {
-        indent: clean_indentation_string(&*indent),
+        indent: clean_indentation_string(&indent),
         percent_bar: bar_text,
         display_data: draw_data.display_data,
     };
@@ -257,7 +281,10 @@ fn pad_or_trim_filename(node: &DisplayNode, indent: &str, display_data: &Display
     let indent_and_name = format!("{} {}", indent, name);
     let width = UnicodeWidthStr::width(&*indent_and_name);
 
-    assert!(display_data.longest_string_length >= width);
+    assert!(
+        display_data.longest_string_length >= width,
+        "Terminal width not wide enough to draw directory tree"
+    );
 
     // Add spaces after the filename so we can draw the % used bar chart.
     let name_and_padding = name
@@ -265,15 +292,19 @@ fn pad_or_trim_filename(node: &DisplayNode, indent: &str, display_data: &Display
             .repeat(display_data.longest_string_length - width)
             .as_str();
 
-    maybe_trim_filename(name_and_padding, display_data)
+    name_and_padding
 }
 
-fn maybe_trim_filename(name_in: String, display_data: &DisplayData) -> String {
-    if UnicodeWidthStr::width(&*name_in) > display_data.longest_string_length {
-        let name = name_in
-            .chars()
-            .take(display_data.longest_string_length - 2)
-            .collect::<String>();
+fn maybe_trim_filename(name_in: String, indent: &str, display_data: &DisplayData) -> String {
+    let indent_length = UnicodeWidthStr::width(indent);
+    assert!(
+        display_data.longest_string_length >= indent_length + 2,
+        "Terminal width not wide enough to draw directory tree"
+    );
+
+    let max_size = display_data.longest_string_length - indent_length;
+    if UnicodeWidthStr::width(&*name_in) > max_size {
+        let name = name_in.chars().take(max_size - 2).collect::<String>();
         name + ".."
     } else {
         name_in
@@ -306,20 +337,19 @@ fn get_name_percent(
         (percents, name_and_padding)
     } else {
         let n = get_printable_name(&node.name, display_data.short_paths);
-        let name = maybe_trim_filename(n, display_data);
+        let name = maybe_trim_filename(n, indent, display_data);
         ("".into(), name)
     }
 }
 
 fn get_pretty_size(node: &DisplayNode, is_biggest: bool, display_data: &DisplayData) -> String {
     let output = if display_data.by_filecount {
-        let size_as_str = node.size.separate_with_commas();
-        let spaces_to_add =
-            display_data.num_chars_needed_on_left_most - size_as_str.chars().count();
-        size_as_str + " ".repeat(spaces_to_add).as_str()
+        node.size.separate_with_commas()
     } else {
-        format!("{:>5}", human_readable_number(node.size))
+        human_readable_number(node.size, display_data.iso)
     };
+    let spaces_to_add = display_data.num_chars_needed_on_left_most - output.chars().count();
+    let output = " ".repeat(spaces_to_add) + output.as_str();
 
     if is_biggest && display_data.colors_on {
         format!("{}", Red.paint(output))
@@ -334,10 +364,10 @@ fn get_pretty_name(
     display_data: &DisplayData,
 ) -> String {
     if display_data.colors_on {
-        let meta_result = fs::metadata(node.name.clone());
+        let meta_result = fs::metadata(&node.name);
         let directory_color = display_data
             .ls_colors
-            .style_for_path_with_metadata(node.name.clone(), meta_result.as_ref().ok());
+            .style_for_path_with_metadata(&node.name, meta_result.as_ref().ok());
         let ansi_style = directory_color
             .map(Style::to_ansi_term_style)
             .unwrap_or_default();
@@ -347,9 +377,10 @@ fn get_pretty_name(
     }
 }
 
-fn human_readable_number(size: u64) -> String {
+fn human_readable_number(size: u64, iso: bool) -> String {
     for (i, u) in UNITS.iter().enumerate() {
-        let marker = 1024u64.pow((UNITS.len() - i) as u32);
+        let num: u64 = if iso { 1000 } else { 1024 };
+        let marker = num.pow((UNITS.len() - i) as u32);
         if size >= marker {
             if size / marker < 10 {
                 return format!("{:.1}{}", (size as f32 / marker as f32), u);
@@ -358,7 +389,7 @@ fn human_readable_number(size: u64) -> String {
             }
         }
     }
-    return format!("{}B", size);
+    format!("{}B", size)
 }
 
 mod tests {
@@ -378,6 +409,7 @@ mod tests {
             base_size: 1,
             longest_string_length,
             ls_colors: LsColors::from_env().unwrap_or_default(),
+            iso: false,
         }
     }
 
@@ -397,7 +429,7 @@ mod tests {
             indent,
             percent_bar,
             is_biggest,
-            &get_fake_display_data(6),
+            &get_fake_display_data(20),
         );
         assert_eq!(s, " 4.0K ┌─┴ short");
     }
@@ -418,21 +450,27 @@ mod tests {
         let s = format_string(&n, indent, percent_bar, is_biggest, &dd);
         assert_eq!(
             s,
-            " 4.0K ┌─┴ very_long_name_longer_than_the_eighty_character_limit_very_lon.."
+            " 4.0K ┌─┴ very_long_name_longer_than_the_eighty_character_limit_very_.."
         );
     }
 
     #[test]
     fn test_human_readable_number() {
-        assert_eq!(human_readable_number(1), "1B");
-        assert_eq!(human_readable_number(956), "956B");
-        assert_eq!(human_readable_number(1004), "1004B");
-        assert_eq!(human_readable_number(1024), "1.0K");
-        assert_eq!(human_readable_number(1536), "1.5K");
-        assert_eq!(human_readable_number(1024 * 512), "512K");
-        assert_eq!(human_readable_number(1024 * 1024), "1.0M");
-        assert_eq!(human_readable_number(1024 * 1024 * 1024 - 1), "1023M");
-        assert_eq!(human_readable_number(1024 * 1024 * 1024 * 20), "20G");
-        assert_eq!(human_readable_number(1024 * 1024 * 1024 * 1024), "1.0T");
+        assert_eq!(human_readable_number(1, false), "1B");
+        assert_eq!(human_readable_number(956, false), "956B");
+        assert_eq!(human_readable_number(1004, false), "1004B");
+        assert_eq!(human_readable_number(1024, false), "1.0K");
+        assert_eq!(human_readable_number(1536, false), "1.5K");
+        assert_eq!(human_readable_number(1024 * 512, false), "512K");
+        assert_eq!(human_readable_number(1024 * 1024, false), "1.0M");
+        assert_eq!(
+            human_readable_number(1024 * 1024 * 1024 - 1, false),
+            "1023M"
+        );
+        assert_eq!(human_readable_number(1024 * 1024 * 1024 * 20, false), "20G");
+        assert_eq!(
+            human_readable_number(1024 * 1024 * 1024 * 1024, false),
+            "1.0T"
+        );
     }
 }
